open Constants;;
(**
 Functions to generate individuals and populations for the genetic algorithm.

 @version December 2003
 @author Pierre Lemaire
*)

open Item;;
open Instance;;
open Gen_sol;;
open Gen_sol_none;;
open Gen_sol_bf;;
open Gen_sol_dc;;
open Gen_pop;;

(**
  [generate_gen_sol inst algo] returns a [gen_sol] that solves the [instance]
  [inst], using the algorithm  [algo].
*)
let generate_gen_sol (inst:instance) (algo:string) =
  let m = inst#get_m()
  and n = inst#get_n()
  and items_inst = ref (inst#get_items())
  in let items_sol = Array.make n (new item 0 0)
  in let rec sel l r n = match (l) with
        | (x::[]) -> (x, r)
        | (x::xs) -> if (Random.int n) == 0 then (x,xs@r) else sel xs (x::r) (n-1)
        | _       -> failwith "BUG !!!"
  in for i=n-1 downto 0 do
       let (x,l) = sel !items_inst [] i
       in items_inst := l;
          items_sol.(i) <- x
     done;
     match algo with
       | "bf" -> (new gen_sol_bf m items_sol :> gen_sol)
       | "dc" -> (new gen_sol_dc m items_sol :> gen_sol)
       | _     -> failwith (Printf.sprintf "error: unknown algorithm %s (abort)\n" algo); 
;;

(**
  [generate_gen _pop p inst algo] returns a [gen_pop] of [p] [gen_sols] that
  solve the [instance] [inst], using the algorithm [algo].
*)
let generate_gen_pop (p:int) (inst:instance) (algo:string) =
  let sols = Array.make p ((new gen_sol_none 0 [||]) :> gen_sol)
  in for i=0 to (Array.length sols)-1 do
       sols.(i) <- generate_gen_sol (inst:instance) (algo:string);
     done;
     new gen_pop sols;
;;
