open Constants;;
(** A [gen_sol] builds according to the best-fit (BF) rule.

 @version December 2003
 @author Pierre Lemaire
 *)

open Options;;
open Item;;
open Bin;;
open Gen_sol;;
open Solution;;
open Solution_bf;;

class gen_sol_bf (m:int) (items:item array) =
object (self)
  inherit gen_sol m items as super

  method get_type () = cGEN_SOL_BF_TYPE;

  method clone () = 
    let it = Array.copy items
    in ((new gen_sol_bf m it) :> gen_sol)

  (** (re-)computes the value of this [gen_sol]. The [items] are packed
      according to the BF rule. *)
  method private compute_value () =
    let n = Array.length items
    and bins = ref (Array.make m (new bin))
    in begin 
       (* -- init: an empty solution -- *)
       for i=0 to m-1 do !bins.(i) <- (new bin) done;
       value <- 0;
       
       (* -- compute BF -- *)
       tracer 2 "......compute value of gen_sol (BF)...";
       for j=0 to (n-1) do
         (* adds item j to the sj lowest (first) bins *)
         for i=0 to items.(j)#get_size -1 do
           !bins.(i)#add items.(j);
           if (value < !bins.(i)#get_height) then value <- !bins.(i)#get_height;
         done;

         (* re-order the bins *)
         let k = ref 0 and l = ref items.(j)#get_size
         and tmp = Array.make m (new bin)
         in for i=0 to m-1 do 
              if (!k >= items.(j)#get_size)
              then begin tmp.(i) <- !bins.(!l); l := !l+1; end
              else if (!l >= m)
              then begin tmp.(i) <- !bins.(!k); k := !k+1; end
              else if (!bins.(!k)#get_height <= !bins.(!l)#get_height)
              then begin tmp.(i) <- !bins.(!k); k := !k+1; end
              else begin tmp.(i) <- !bins.(!l); l := !l+1; end;
            done;
            bins := tmp;
       done;
       tracer 2 (Printf.sprintf "value= %d\n" value);
       end

  method to_solution () =
    let inst = self#to_instance()
    in let sol = (new solution_bf (inst#get_m()) :> solution); 
       in sol#build inst; 
          sol

end
