open Constants;;
(** A [gen_sol] builds according to the DC algorithm.

 @version December 2003
 @author Pierre Lemaire
 *)

open Options;;
open Item;;
open Bin;;
open Gen_sol;;
open Solution;;
open Solution_dc;;

class gen_sol_dc (m:int) (items:item array) =
object (self)
  inherit gen_sol m items as super

  method get_type () = cGEN_SOL_DC_TYPE;

  method clone () = 
    let it = Array.copy items
    in ((new gen_sol_dc m it) :> gen_sol)


  (** (re-)computes the value of this [gen_sol]. The [items] are packed
      according to the DC algorithm. *)
  method private compute_value () =
    let n = Array.length items
    in (* -- init -- *)
       let tmp = Array.create_matrix n m (new bin)
       in for s=0 to n-1 do for b=0 to m-1 do tmp.(s).(b) <- new bin ; done; done;

       (* -- compute DC -- *)
       tracer 2 "......compute value of gen_sol (DC)...";
       (* builds n partial solutions *)
       for j=0 to n-1 do
         for i=0 to items.(j)#get_size-1 do
           tmp.(j).(i)#add items.(j);
         done;
       done;

       (* merges solutions *)
       let k = ref 1 and s = ref 1
       in while (!k<n) do 
         s := 0;
         while (!s<n) do
           if (!s + !k < n) then begin
             for i=0 to m-1 do
               tmp.(!s).(i)#append tmp.(!s + !k).(m-1-i);
             done;
             Array.fast_sort (compare_bin) tmp.(!s);
           end;
           s := !s + 2 * !k;
         done;
         k := !k * 2;
       done;

       (* keeps last solution *)
       let max = ref tmp.(0).(0)#get_height
       in for i=1 to (Array.length tmp.(0) -1) do
            if (!max < tmp.(0).(i)#get_height) then max := tmp.(0).(i)#get_height
          done;
          value <- !max;
          tracer 2 (Printf.sprintf " value = %d\n" value);


  method private to_solution () =
    let inst = self#to_instance()
    in let sol = (new solution_dc (inst#get_m()) :> solution); 
       in sol#build inst; 
          sol

end
