open Constants;;
(**
 A generator for multibin instances. See help for details.

 @version December 2003
 @author Pierre Lemaire
*)

(* ========================================================================= *)
(* ========================================================================= *)

open Options;;
open Instance;;

let generator() =
  let usage_msg = "available options are:"
  and m = ref 5
  and n = ref 50
  and hmin = ref 1
  and hmax = ref 17
  and vh   = ref (-1)
  and smin = ref 1
  and smax = ref 4
  and vs   = ref (-1)
  and seed = ref (-1)
  and algo = ref cGEN_RAND
  and file = ref ""
  in let set_id id v = id := v
     and set_algo x (y:unit) = algo := x
  in let arglist =
    [ ("-f",       Arg.String (set_id file), "(name) sets the output file name (default is stdout)");
      ("-n",       Arg.Int (set_id n),    "(int) sets the number of items.");
      ("-m",       Arg.Int (set_id m),    "(int) sets the number of bins.");
      ("-hmin",    Arg.Int (set_id hmin), "(int) sets the minimal possible height for an item.");
      ("-hmax",    Arg.Int (set_id hmax), "(int) sets the maximal possible height for an item (1 <= hmin <= hmax).");
      ("-vh",      Arg.Int (set_id vh),   "(int) sets the variance for the heights (gaussian generation).");
      ("-smin",    Arg.Int (set_id smin), "(int) sets the minimal possible size for an item.");
      ("-smax",    Arg.Int (set_id smax), "(int) sets the maximal possible size for an item (1 <= smin <= smax <= m).");
      ("-vs",      Arg.Int (set_id vs),   "(int) sets the variance for the sizes (gaussian generation).");
      ("-seed",    Arg.Int (set_id seed), "(int) sets the seed for random numbers.");
      ("-rand",       Arg.Unit (set_algo cGEN_RAND),  "items are generated uniformaly in [hmin;hmax],[smin;smax].");
      ("-gauss",      Arg.Unit (set_algo cGEN_GAUSS), "items are generated normaly, with variance vh in [hmin;hmax] and vs in [smin;smax].");
      ("-fixedV",     Arg.Unit (set_algo cGEN_FIXED_V), "items are of fixed height hmax and fixed size smax.");
      ("-fixedS",     Arg.Unit (set_algo cGEN_FIXED_S), "items are of random height, fixed size.");
      ("-fixedH",     Arg.Unit (set_algo cGEN_FIXED_H), "items are of random size, fixed height.");
      ("-stairsV",    Arg.Unit (set_algo cGEN_STAIRS_V), "h = h(mod hmax)+hmin; s = s(mod smax)+smin.");
      ("-stairsS",    Arg.Unit (set_algo cGEN_STAIRS_S), "height is random; s = s(mod smax)+smin.");
      ("-stairsH",    Arg.Unit (set_algo cGEN_STAIRS_H), "h = h(mod hmax)+hmin; size is random.");
      ("-complement", Arg.Unit (set_algo cGEN_COMPLEMENT), "items are created by pair of complentary sizes.");
      ("-difference", Arg.Unit (set_algo cGEN_DIFFERENCE), "idem, and they are of egal height.");
      ("-display", Arg.String (set_display), "[simple|normal|pretty] sets the mode for displaying items (default is 1)");
    ]
  in

    Arg.parse arglist (fun s -> ()) usage_msg;
    let inst = 
      try generate_instance !m !n !hmin !hmax !vh !smin !smax !vs !algo !seed
      with Invalid_argument msg -> Printf.printf "%s\n" msg; exit(1);
    and out =
      if (!file = "") then stdout
      else
        try open_out !file
        with Sys_error msg ->
          Printf.fprintf stderr "%s\n" msg;
          stdout;
    in 
       Printf.fprintf out "%s\n" (inst#print());
       close_out out;
       exit 0
;;

(* ========================================================================= *)
if !Sys.interactive
then
  ()
else begin
  generator()
end;;

(* ========================================================================= *)
(* ========================================================================= *)
