open Constants;;
(**
 A genetic algorithm to solve multibin problems.

 @version December 2003
 @author Pierre Lemaire
*)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;
open Gen_pop;;
open Gen_gen;;

(** 
  
 *)
let genetic inst algo pop nb_gen
            crossrate mutaterate
            fitness fitmax fitdecr
            reproduction elitism steadytries =
  tracer 0 "Genetic algorithm...\n";
  let pop = tracer 1 "...init genetics\n";
            generate_gen_pop pop inst algo
  in let bestval = ref (pop#best_value())
  and lowerbound = inst#get_bound()
  and bestiter = ref 0
  and iter = ref 1
  in if (crossrate  >= 0.0) then begin pop#set_cross_rate crossrate; end;
     if (mutaterate >= 0.0) then begin pop#set_mutate_rate mutaterate; end;
     if (fitness    <>  "") then begin pop#set_fitness fitness; end;
     if (fitmax     >=   0) then begin pop#set_max_fitness  fitmax; end;
     if (fitdecr    >=   0) then begin pop#set_decr_fitness fitdecr; end;
     if (steadytries>=   0) then begin pop#set_steady_tries steadytries; end;
     if (reproduction <>  "") then begin pop#set_reproduction reproduction; end;
     pop#set_elitism elitism;
     tracer 1 (Printf.sprintf "...generation %3d; best found so far: %d\n" 0 (pop#best_value()));
     if (!bestval <= lowerbound) then tracer 0 (Printf.sprintf "...opt found at iter 0 of genetics!\n")
     else while (!iter <= nb_gen) do
       pop#next_generation();
       tracer 1 (Printf.sprintf "...generation %3d; best found so far: %d\n" !iter (pop#best_value()));
       if (pop#best_value() < !bestval) then begin
         bestval := pop#best_value();
         bestiter := !iter;
         if (!bestval <= lowerbound) then begin
           tracer 0 (Printf.sprintf "...opt found at iter %d of genetics!\n" !iter);
           iter := nb_gen;
         end
       end;
       iter := !iter + 1
     done;
     tracer 0 (Printf.sprintf "...done with genetics (best is %d, found at iter %d / %d)\n" !bestval !bestiter (!iter - 1));
     pop#best_solution();
;;
