open Constants;;
open Options;;
open Item;;
open Bin;;
open Myrand;;

open Scanf;;

(* hidden *)
let print_item (x:item) = x#print

class instance (nb_m:int) =
object(self)
  val m = nb_m;
  val mutable value = 0;
  val mutable n = 0;
  val mutable hmax = 0;
  val mutable smax = 0;
  val mutable items = [];

  method get_n () = n;
  method get_m () = m;
  method get_hmax () = hmax;
  method get_smax () = smax;

  method get_hmin () = 
    let rec f x l = match l with
          | [] -> x
          | (y::ys) -> f (min y#get_height x) ys
    in f hmax items 

  method get_smin () = 
    let rec f x l = match l with
          | [] -> x
          | (y::ys) -> f (min y#get_size x) ys
    in f smax items 


  method get_volume () = 
    let rec vol_total x l = match l with
      | (y::ys) -> vol_total (x + y#get_volume) ys
      | _       -> x
    in vol_total 0 items

  method get_bound () = 
    let rec vol_total x l = match l with
      | (y::ys) -> vol_total (x + y#get_volume) ys
      | _       -> x
    in max hmax (int_of_float (ceil (float_of_int(self#get_volume()) /. float_of_int(m))))

  method add (x:item) =
    items <- x::items;
    n <- n+1;
    if (hmax < x#get_height) then hmax <- x#get_height;
    if (smax < x#get_size)   then smax <- x#get_size;

  method get_items () = items;

  method sort_items () =
    tracer 0 "Sorting objects...\n";
    items <- List.sort (compare_item) items;

  method print () =
    (string_of_int m)^"\n"^(string_of_int n)^"\n"^(String.concat "" (List.map print_item items))
end

let generate_instance mx nx hminx hmax vhx sminx smaxx vsx algo seed =
  let n = if (nx < 1) then 1 else nx
  and m = if (mx < 1) then 1 else mx
  in let hmin = if (hminx < 1) then 1 else hminx
     and smin = if (sminx < 1) then 1 else sminx
     and smax = if (smaxx > m) then m else smaxx
  in let hmin = if (hmin > hmax) then hmax else hmin
     and smin = if (smin > smax) then smax else smin
  in let vh = if (vhx < 0) then 3*(hmax-hmin) else vhx
     and vs = if (vsx < 0) then 3*(smax-smin) else vsx
  in if (seed < 0) then Random.self_init () else Random.init seed;
     let inst = new instance m
     in match (algo) with
        | cGEN_RAND ->
            for j=1 to n do
              inst#add (new item (hmin+Random.int (1+hmax-hmin)) (smin+Random.int (1+smax-smin))) done;
            inst;
        | cGEN_GAUSS ->
            for j=1 to n do
              inst#add (new item (Myrand.truncated_gauss hmin hmax vh) (Myrand.truncated_gauss smin smax vs)) done;
            inst;
        | cGEN_FIXED_V ->
            for j=1 to n do
              inst#add (new item hmax smax) done;
            inst;
        | cGEN_FIXED_H ->
            for j=1 to n do
              inst#add (new item hmax (smin+Random.int (1+smax-smin))) done;
            inst;
        | cGEN_FIXED_S ->
            for j=1 to n do
              inst#add (new item (hmin+Random.int (1+hmax-hmin)) smax) done;
            inst;
        | cGEN_STAIRS_H ->
          let h = ref hmin
          in for j=1 to n do
               inst#add (new item !h (smin+Random.int (1+smax-smin)));
      	       if (!h < hmax) then h := !h + 1 else h := hmin;
             done;
             inst;
        | cGEN_STAIRS_S ->
          let s = ref smin
          in for j=1 to n do
               inst#add (new item (hmin+Random.int (1+hmax-hmin)) !s);
      	       if (!s < smax) then s := !s + 1 else s := smin;
             done;
             inst;
        | cGEN_STAIRS_V ->
          let h = ref hmin
          and s = ref smin
          in for j=1 to n do
               inst#add (new item !h !s);
      	       if (!h < hmax) then h := !h + 1 else h := hmin;
      	       if (!s < smax) then s := !s + 1 else s := smin;
             done;
             inst;
        | cGEN_DIFFERENCE ->
          if (m < 2) then raise(Invalid_argument "generate_instance requires m >= 2 for cGEN_COMPLEMENT");
          let h = ref 1 and s = ref 1
          in for j=1 to n do
               if (j mod 2 == 1) then begin
                 h := hmin+Random.int (1+hmax-hmin);
      	         s := 1+Random.int(m / 2);
               end
               else s := m - !s;
               inst#add (new item !h !s);
             done;
             inst;
        | cGEN_COMPLEMENT ->
          if (m < 2) then raise(Invalid_argument "generate_instance requires m >= 2 for cGEN_COMPLEMENT");
          let s = ref 1
          in for j=1 to n do
               if (j mod 2 == 1) then s := (1+Random.int (m / 2))
               else s := m - !s;
               inst#add (new item (hmin+Random.int (1+hmax-hmin)) !s);
             done;
             inst;
        | _ -> raise(Invalid_argument "generate_instance: not a valid method");
;;

let read_instance fname =
  let input =
    if (fname = "") then stdin
    else try open_in fname
         with Sys_error msg ->
                Printf.fprintf stderr "warning: can't open %s (stdin used instead)\n" fname;
                stdin
  in try 
    let m = fscanf input " %d" (fun x -> x)
    and n = fscanf input " %d" (fun x -> x)
    in
      let inst = new instance m
      in for i=0 to n-1 do
           let h = fscanf input " %d" (fun x -> x)
           and s = fscanf input " %d" (fun x -> x)
           in inst#add (new item h s)
         done;
         inst;
  with
  | Scan_failure msg ->
      Printf.fprintf stderr "error: wrong format (abort)\n";
      exit 1
  | Failure msg ->
      Printf.fprintf stderr "error: wrong format (abort)\n";
      exit 1
  | End_of_file ->
      Printf.fprintf stderr "error: wrong format (abort)\n";
      exit 1
;;
