open Constants;;
open Options;;

(* hidden auxilaries *)
let next_color = ref 32
let get_next_color () =
  next_color := (!next_color + 1);
  if (!next_color > 255) then next_color := 32;
  Char.chr !next_color

class item (h: int) (s:int) =
object
  val height = h;
  val size = s;
  val color = get_next_color()
  method get_height = height;
  method get_size   = size;
  method get_volume = height * size;
  method print = match (options.display) with
   | cDISPLAY_SIMPLE -> (string_of_int height)^" "^(string_of_int size)^" ";
   | cDISPLAY_NORMAL -> "("^(string_of_int height)^","^(string_of_int size)^")";
   | _              -> String.make height color;

end

(* hidden comparison functions *)
let compare_item_0 (x:item) (y:item) = 0
let compare_item_1 (x:item) (y:item) = y#get_height - x#get_height
let compare_item_2 (x:item) (y:item) = x#get_height - y#get_height
let compare_item_3 (x:item) (y:item) = y#get_size - x#get_size
let compare_item_4 (x:item) (y:item) = x#get_size - y#get_size
let compare_item_5 (x:item) (y:item) = y#get_volume - x#get_volume
let compare_item_6 (x:item) (y:item) = x#get_volume - y#get_volume
let compare_item_7 (x:item) (y:item) = (Random.int 11) - 5
let compare_item_f1 = ref compare_item_1
let compare_item_f2 = ref compare_item_0

let compare_item (x:item) (y:item) =
  let cmp = !compare_item_f1 x y
  in if (cmp=0) then !compare_item_f2 x y else cmp

let set_compare_item1 i =
  compare_item_f1 := match i with
  |  x when(x=cORDER_DECR+cORDER_H) -> compare_item_1
  |  x when(x=cORDER_INCR+cORDER_H) -> compare_item_2
  |  x when(x=cORDER_DECR+cORDER_S) -> compare_item_3
  |  x when(x=cORDER_INCR+cORDER_S) -> compare_item_4
  |  x when(x=cORDER_DECR+cORDER_V) -> compare_item_5
  |  x when(x=cORDER_INCR+cORDER_V) -> compare_item_6
  |  x when(x=cORDER_ALEA)         -> compare_item_7
  |  _ -> raise(Invalid_argument "set_compare_item: not a valid function")

let set_compare_item2 i =
  compare_item_f2 := match i with
  |  x when(x=cORDER_DECR+cORDER_H) -> compare_item_1
  |  x when(x=cORDER_INCR+cORDER_H) -> compare_item_2
  |  x when(x=cORDER_DECR+cORDER_S) -> compare_item_3
  |  x when(x=cORDER_INCR+cORDER_S) -> compare_item_4
  |  x when(x=cORDER_DECR+cORDER_V) -> compare_item_5
  |  x when(x=cORDER_INCR+cORDER_V) -> compare_item_6
  |  x when(x=cORDER_ALEA)         -> compare_item_7
  |  _                            -> compare_item_0

let set_compare_item s =
  let dir = match (Char.lowercase s.[0]) with
    | 'i' -> cORDER_INCR
    | 'd' -> cORDER_DECR
    | 'a' -> cORDER_ALEA
    | _   -> raise (Invalid_argument "invalid first direction")
  in begin
    if (dir = cORDER_ALEA) then set_compare_item1 cORDER_ALEA
    else 
      begin match (Char.lowercase s.[1]) with
        | 'h' -> set_compare_item1 (dir+cORDER_H);
        | 's' -> set_compare_item1 (dir+cORDER_S);
        | 'v' -> set_compare_item1 (dir+cORDER_V);
        | _   -> raise (Invalid_argument "invalid first criterion");
       end ;
    if (String.length s > 2) then
        let dir = match (Char.lowercase s.[2]) with
          | 'i' -> cORDER_INCR
          | 'd' -> cORDER_DECR
          | 'a' -> cORDER_ALEA
          | _   -> raise (Invalid_argument "invalid second direction")
        in
          if (dir = cORDER_ALEA) then set_compare_item2 cORDER_ALEA
          else match (Char.lowercase s.[3]) with
            | 'h' -> set_compare_item2 (dir+cORDER_H)
            | 's' -> set_compare_item2 (dir+cORDER_S)
            | 'v' -> set_compare_item2 (dir+cORDER_V)
            | _   -> raise (Invalid_argument "invalid first criterion")
    end
