(**
 The [item] class, which represents a multibin object of fixed size and height.

 @version November 2003
 @author Pierre Lemaire
*)

(** [item h s] creates a new [item] of height [h] and of size [s]. *)
class item:
  int -> (* height *)
  int -> (* size *)
object

  (** @return the height of this [item]. *)
  method get_height: int

  (** @return the size of this [item]. *)
  method get_size: int

  (** @return the volume of this [item]. *)
  method get_volume: int

  (** @return a string-representation of this [item]. *)
  method print: string
end

(** [compare_item x y] returns [0] if [x=y], a negative integer if [x<y], and 
    a positive integer if [x>y]. *)
val compare_item: item -> item -> int

(** sets the first comparison function used for [compare_item]. Possible
    directions are:
{ul {- ORDER_DECR: decreasing}    {- ORDER_INCR: increasing}  {- ORDER_ALEA: random }} 
    and possible criteria are:
{ul {- ORDER_H: height}    {- ORDER_S: size}  {- ORDER_V: volume }} 
    e.g. uses ORDER_DECR+ORDER_H for decreasing height.

    @raise Invalid_argument if the order is unkown.
*)
val set_compare_item1: int -> unit

(** sets the second comparison function used for [compare_item]. Possible
    directions are:
{ul {- ORDER_DECR: decreasing}    {- ORDER_INCR: increasing}  {- ORDER_ALEA: random }} 
    and possible criteria are:
{ul {- ORDER_H: height}    {- ORDER_S: size}  {- ORDER_V: volume }} 
    e.g. uses ORDER_DECR+ORDER_H for decreasing height.

    @raise Invalid_argument if the order is unkown.
*)
val set_compare_item2: int -> unit

(** sets the comparison functions used for [compare_item]. The format of the
    [string] is [a|(i|d)(h|s|v)[a|(i|d)(h|s|v)]] *)
val set_compare_item: string -> unit
