open Constants;;
(** Defines several generic functions. 

 @version December 2003
 @author Pierre Lemaire
*)

open Options;;

(** [set_id id v] sets [id] to the value [v] *)
let set_id id v =
  id := v

(** [open_file fname alt] returns the [out_channel] associated to the file
    [fname] if it can be opened in write mode and returns [alt] otherwise. *)
let open_file fname alt =
  if (fname="") then alt
  else
    try open_out fname
    with Sys_error msg ->
      Printf.fprintf stderr "warning: %s (%s used for printing times)\n" msg (if (alt=stdout) then "stdout" else if (alt=stderr) then "stderr" else "???");
      alt
;;


(** [pretty_print_int out cmt val] prints the value [val] on [out], preceded
    by the comment [cmt], if the display is set to "pretty" *)let pretty_print_int out pretty value =
  if (options.display=cDISPLAY_PRETTY) then
    Printf.fprintf out "%s%d\n" pretty value
  else
    Printf.fprintf out "%d\n" value
;;
