open Constants;;
(** Defines several functions to generate particular random numbers.

 @version December 2003
 @author Pierre Lemaire
 *)


(** [gauss m s] returns a gaussian number of mean [m] and of variance
    [s^2]. *)
let gauss m s =
  let a = Random.float 1.0
  and b = Random.float 1.0
  and fm = float_of_int m
  and fs = float_of_int s
  in int_of_float(fm +. fs *. (cos (2.0*.cPI*.a))*.(sqrt (-2.0*.(log b))))
;;

(** [truncated_gauss a b s] returns a gaussian number of mean [(a+b)/2] and
    of variance [s^2], truncated in \[[a;b]\]. *)
let truncated_gauss a b s =
  let m = (a+b)/2
  in let x = ref (gauss m s)
     in while (!x < a) || (!x > b) do
        x := (gauss m s);
        done;
        !x
;;

(** [lognorm m s] returns a log-normal number of mean [m] and of variance
    [s^2]. ************* a faire ******************
*)

(** [permuation n] returns a random permutation of \{0,1,...n-1\}. *)
let permutation n =
  let rec make_perm n = match (n) with
        | 1 -> [0]
        | _ -> (n-1)::(make_perm (n-1))
  in let rec sel l r n = match (l) with
        | (x::[]) -> (x, r)
        | (x::xs) -> if (Random.int n) == 0 then (x,xs@r) else sel xs (x::r) (n-1)
        | _       -> failwith "BUG !!!"
  in let perm_array = Array.make n (0)
     and perm_list  = ref (make_perm n)
  in for i=(n-1) downto 0 do
       let (x,l) = sel !perm_list [] (i+1)
       in perm_list := l;
          perm_array.(i) <- x
     done;
     perm_array
;;
