open Constants;;
(**
  Generate random arguments for the instance generator.

 @version December 2003
 @author Pierre Lemaire
*)

(* ========================================================================= *)
(* ========================================================================= *)

let randargs() =
  let usage_msg = "Generate random arguments for the instance generator.\nAvailable options are:"
  and mmin = ref 2
  and mmax = ref 20
  and nmin = ref 2
  and nmax = ref 1000
  and hmin = ref 50
  and hmax = ref 50
  and vh   = ref 50
  and smin = ref 50
  and smax = ref 50
  and vs   = ref 50
  and seed = ref (-1)
  in let set_id id v = id := v
  in let arglist =
    [ ("-nmin",    Arg.Int (set_id nmin), "(int) sets the minimal number of items.");
      ("-nmax",    Arg.Int (set_id nmax), "(int) sets the maximal number of items.");
      ("-mmin",    Arg.Int (set_id mmin), "(int) sets the minimal number of bins.");
      ("-mmax",    Arg.Int (set_id mmax), "(int) sets the maximal number of bins.");
      ("-hmin",    Arg.Int (set_id hmin), "(int) sets the minimal possible height for an item.");
      ("-hmax",    Arg.Int (set_id hmax), "(int) sets the maximal possible height for an item (1 <= hmin <= hmax).");
      ("-vh",      Arg.Int (set_id vh),   "(int) sets the variance for the heights (gaussian generation).");
      ("-smin",    Arg.Int (set_id smin), "(int) sets the minimal possible size for an item.");
      ("-smax",    Arg.Int (set_id smax), "(int) sets the maximal possible size for an item (1 <= smin <= smax <= m).");
      ("-vs",      Arg.Int (set_id vs),   "(int) sets the variance for the sizes (gaussian generation).");
      ("-seed",    Arg.Int (set_id seed), "(int) sets the seed for random numbers.");
    ]
  in
    Arg.parse arglist (fun s -> ()) usage_msg;
    if (!seed = -1) then Random.self_init() else Random.init !seed;
    (* sets values *)
    let m    = !mmin+(Random.int (1 + !mmax - !mmin))
    and n    = !nmin+(Random.int (1 + !nmax - !nmin))
    and hmin = 1+(Random.int (max 1 !hmin))
    and hmax = 1+(Random.int (max 1 !hmax))
    and vh   = 1+(Random.int (max 1 !vh))
    and smin = 2+(Random.int (max 1 !smin))
    and smax = 2+(Random.int (max 1 !smax))
    and vs   = 1+(Random.int (max 1 !vs))
    and algo = Random.int 100
    (* corrects 'errors' *)
    in let hmin, hmax = min hmin hmax, max hmin hmax
       and smin = min smin m
       and smax = min smax m
       in let smin, smax = min smin smax, max smin smax
    (* prints *)
    in if (!seed = -1) then Printf.printf "" else Printf.printf "-seed %d " !seed ;
       Printf.printf "-n %.3d -m %.2d -hmin %.2d -hmax %.2d -vh %.2d -smin %.2d -smax %.2d -vs %.2d %s"
         n m hmin hmax vh smin smax vs
         (if (algo < 41) then "-rand"
          else if (algo < 82) then "-gauss"
(*          else if (algo < 82) then "-fixedV"*)
          else if (algo < 85) then "-fixedS"
(*          else if (algo < 85) then "-fixedH"*)
          else if (algo < 88) then "-stairsV"
          else if (algo < 91) then "-stairsS"
          else if (algo < 94) then "-stairsH"
          else if (algo < 97) then "-complement"
          else                     "-difference");
       exit 0
;;

(* ========================================================================= *)
if !Sys.interactive
then
  ()
else begin
  randargs()
end;;

(* ========================================================================= *)
(* ========================================================================= *)
