open Constants;;
(** A [solution] builds according to the alea divide and conquer (ADC)
    algortithm. That is the best solution among several DC solutions, for
    different random ordering of the [items]. 

 @version December 2003
 @author Pierre Lemaire
*)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;
open Solution_dc;;

(** [solution_adc m t] creates a new empty [solution_adc] of [m bins], and with
a number of tries [t]. *)
class solution_adc (m:int) (t:int) =
object (self)
  inherit solution_dc m as super
  val mutable nb_tries = t

  (** sets the number of tries for the [build] method. *)
  method set_nb_tries x = nb_tries <- x; 

  (** performs a DC on the given [instance] ; then re-order the [items]
      randomly and perform a BF algorithm with this new order. The second step
      is done until [nb_tries] [solutions] have been build, and the best one
      is kept (note that the order of the [items] in the [instance] is changed).
  *)
  method build (inst:instance) =
    tracer 0 "Performing Alea DC...\n";
    super#build inst;
    let bestval = ref value
    and bestbin = ref bins
    in set_compare_item "a";
       for i=2 to nb_tries do
         inst#sort_items();
         bins <- Array.make m (new bin);
         for i=0 to m-1 do bins.(i) <- new bin done;
         value <- 0;
         volume <- 0;
         super#build inst;
         tracer 1 ("...solution value is "^string_of_int(value)^"\n");
         if (value < !bestval) then begin
           bestval := value;
           bestbin := bins;
         end
       done;
       bins <- !bestbin;
       value <- !bestval;
end
