open Constants;;
(** A [solution] builds according to the best-fit (BF) rule.

 @version December 2003
 @author Pierre Lemaire
 *)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;

class solution_bf (m:int) =
object (self)
  inherit solution m as super

  (** pack the [items] of the given [instance] according to the BF rule, in the
      order they are in the [instance]. *)
  method build (inst:instance) =
    let items = Array.of_list (inst#get_items())
    and n = inst#get_n()
    and m = inst#get_m()
    in tracer 0 "Performing BF...\n";
       if (m != Array.length bins) then raise(Incompatible_solution);
       for j=0 to (n-1) do
         (* adds item j to the sj lowest (first) bins *)
         for i=0 to items.(j)#get_size -1 do super#add items.(j) i; done;

         (* re-order the bins *)
         let k = ref 0 and l = ref items.(j)#get_size
         and tmp = Array.make m (new bin)
         in for i=0 to m-1 do 
              if (!k >= items.(j)#get_size)
              then begin tmp.(i) <- bins.(!l); l := !l+1; end
              else if (!l >= m)
              then begin tmp.(i) <- bins.(!k); k := !k+1; end
              else if (bins.(!k)#get_height <= bins.(!l)#get_height)
              then begin tmp.(i) <- bins.(!k); k := !k+1; end
              else begin tmp.(i) <- bins.(!l); l := !l+1; end;
            done;
            bins <- tmp;
       done;

end
