open Constants;;
(** A [solution] builds according to the decreasing best-fit (DBF) rule.

 @version December 2003
 @author Pierre Lemaire
*)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;
open Solution_bf;;

class solution_dbf (m:int) =
object (self)
  inherit solution_bf m as super

  (** @return the best known lower bound for this [solution]. \[Unless
      redefined, this bound is the best between the volume-bound, hmax, 
      the bound given by H(S) <= H(OPT) + (m-1)/m * Delta and the bound
      given by H(DBF) <= 4*H(OPT)/3.\] *)
  method get_bound () =
    max (super#get_bound())
        (int_of_float (ceil (float_of_int(value)*.3.0/.4.0)))

  (** re-order the [items] of the [instance] by decreasing height and then 
      pack them according to the BF rule (note that the order of the [items]
      in the [instance] is thus changed). *)
  method build (inst:instance) =
    set_compare_item "dh";
    inst#sort_items();
    super#build inst;
end
