open Constants;;
(** A [solution] builds according to the divide & conquer (DC) algorithm.

 @version December 2003
 @author Pierre Lemaire
*)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;

class solution_dc (m:int) =
object (self)
  inherit solution m as super

  (** [append x i] appends [bin] [x] to the [i]th [bin] of this solution. *)
  method private append (x:bin) (i:int) =
    bins.(i)#append x;
    if (value < bins.(i)#get_height) then value <- bins.(i)#get_height;

  (** pack the [items] of the given [instance] according to the DC algorithms ; 
      [items] are consider in the order they appear in the [instance]. *)
  method build (inst:instance) =
    let items = Array.of_list (inst#get_items())
    and n = inst#get_n()
    and m = inst#get_m()
    and tmphmax = ref 0
    and tmpvol  = ref 0
    in tracer 0 "Performing DC...\n";
       if (m != Array.length bins) then raise(Incompatible_solution);
       let tmp = Array.create_matrix n m (new bin)
       in for s=0 to n-1 do for b=0 to m-1 do tmp.(s).(b) <- new bin ; done; done;

          (* builds n partial solutions *)
          for j=0 to n-1 do
            tmpvol := !tmpvol + items.(j)#get_volume;
            tmphmax := max (!tmphmax) (items.(j)#get_height);
            for i=0 to items.(j)#get_size-1 do
              tmp.(j).(i)#add items.(j);
              done;
          done;

          (* merges solutions *)
          let k = ref 1 and s = ref 1
          in while (!k<n) do 
            s := 0;
            while (!s<n) do
              if (!s + !k < n) then begin
                for i=0 to m-1 do
                  tmp.(!s).(i)#append tmp.(!s + !k).(m-1-i);
                done;
                Array.fast_sort (compare_bin) tmp.(!s);
              end;
              s := !s + 2 * !k;
            done;
            k := !k * 2;
          done;

          (* keeps last solution *)
          bins <- tmp.(0);
          let max = ref 0
          in for i=0 to (Array.length bins -1) do
            if (!max < bins.(i)#get_height) then max := bins.(i)#get_height
            done;
            value <- !max;
            hmax  <- !tmphmax;
            volume <- !tmpvol;

end
