open Constants;;
(** A [solution] builds according to the optimist-placement with varations
   (OPV) rule.

 @version April 2004
 @author Pierre Lemaire
 *)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;

class solution_opv (m:int) =
object (self)
  inherit solution m as super

  (** pack the [items] of the given [instance] according to the BF rule, in the
      order they are in the [instance]. *)
  method build (inst:instance) =
    let items = Array.of_list (inst#get_items())
    and n = inst#get_n()
    and m = inst#get_m()
    and href = ref (int_of_float (ceil ((float_of_int (inst#get_volume())) /. (float_of_int m))))
    in tracer 0 "Performing OPV...\n";
       if (m != Array.length bins) then raise(Incompatible_solution);
       for j=0 to (n-1) do
       let used = Array.make (inst#get_m()) false
       in for k=1 to items.(j)#get_size do
          let ok = ref false
          in for i=0 to (m-1) do 
               if (not !ok) && (not used.(i)) &&
                  (bins.(i)#get_height + items.(j)#get_height <= !href)
               then begin
                 super#add items.(j) i;
                 used.(i) <- true; ok := true;
               end;
             done;
             if not !ok then begin (* can't be packed under Href *)
             let a = ref max_int and b = ref 0
             in for i=0 to (m-1) do
                  if not used.(i) && bins.(i)#get_height < !a then begin
                    a := bins.(i)#get_height; b := i; 
                  end;
                done;
                super#add items.(j) !b;
                href := bins.(!b)#get_height;
                used.(!b) <- true;
             end
          done
       done
end
