open Constants;;
(** A [solution] builds according to the optimist-placement with multiple tries
   (OPX) rule,

 @version April 2004
 @author Pierre Lemaire
 *)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Solution;;

class solution_opx (m:int) =
object (self)
  inherit solution m as super

  (** pack the [items] of the given [instance] according to the BF rule, in the
      order they are in the [instance]. *)
  method build (inst:instance) =
    let items = Array.of_list (inst#get_items())
    and n = inst#get_n()
    and m = inst#get_m()
    and h0 = (int_of_float (ceil ((float_of_int (inst#get_volume())) /. (float_of_int m))))
    and bestval = ref max_int
    and bestbin = ref bins
    in tracer 0 "Performing OPX...\n";
       if (m != Array.length bins) then raise(Incompatible_solution);
       for href=(max (h0-inst#get_hmax()) 1) to h0+inst#get_hmax() do
         bins <- Array.make m (new bin);
         value <- 0;
         for i=0 to m-1 do bins.(i) <- new bin done;
         for j=0 to (n-1) do
         let used = Array.make m false
         in for k=1 to items.(j)#get_size do
            let ok = ref false
            in for i=0 to (m-1) do 
                 if (not !ok) && (not used.(i)) &&
                    (bins.(i)#get_height + items.(j)#get_height <= href)
                 then begin
                   super#add items.(j) i;
                   used.(i) <- true; ok := true;
                 end;
               done;
               if not !ok then begin (* can't be packed under Href *)
               let a = ref max_int and b = ref 0
               in for i=0 to (m-1) do
                    if not used.(i) && bins.(i)#get_height < !a then begin
                      a := bins.(i)#get_height; b := i; 
                    end;
                  done;
                  super#add items.(j) !b;
                  used.(!b) <- true;
               end
            done
         done;
         tracer 1 ("...solution value is "^string_of_int(value)^"\n");
         if (value < !bestval) then begin
           bestval := value;
           bestbin := bins;
         end
       done;
       bins <- !bestbin;
       value <- !bestval;
end
