open Constants;;
(** A [solution] builds according to the permutation best-fit (PBF) algortithm.
    Every permutation is tried, and the best one is returned. Stops if the
    volume-bound is reached.

 @version February 2004
 @author Pierre Lemaire
*)

open Options;;
open Item;;
open Bin;;
open Instance;;
open Instance_perm;;
open Solution;;
open Solution_bf;;

(** [solution_pbf m t] creates a new empty [solution_pbf] of [m bins]. *)
class solution_pbf (m:int) =
object (self)
  inherit solution_bf m as super

  (** ... (note that the order of the [items] in the [instance] is changed).
  *)
  method build (inst:instance) =
    tracer 0 "Performing Permutation BF...\n";
    let instp = new instance_perm(inst);
    in super#build (instp :> instance);
    let bestval = ref value
    and bestbin = ref bins
    and lb = inst#get_bound()
    in while(lb < !bestval && instp#next()) do
         bins <- Array.make m (new bin);
         for i=0 to m-1 do bins.(i) <- new bin done;
         value <- 0;
         volume <- 0;
         super#build (instp :> instance);
         tracer 1 ("...solution value is "^string_of_int(value)^" (best="^string_of_int(!bestval)^")\n");
         if (value < !bestval) then begin
           bestval := value;
           bestbin := bins;
         end
       done;
       bins <- !bestbin;
       value <- !bestval;
end
